#!/usr/bin/env python

import pygame
from pygame.locals import *
import sys
import random

image_files = ["aargh.png", "glen.png", "schultmc.png", "jdr.png", "bob.png", "tottinge.png", "ben.png", "sam.png"] #,"logo1.png"]
loading_file = "loading.png"
music = "popcorn.ogg"
giggle = "giggles.ogg"
variants = [
    'Original (default)',
    "Variant #1 'tottinge butthole'",
    "Variant #2 'zoom trails'",
    "Variant #3 'slow spiral'",
    "Variant #4 'amazing technicolor dream choad'",
    "Variant #5 'toilet flush'",
    "Variant #6 'technicolor flush'",
    "Variant #7 'supa-fgij smash brothers'",
    ]


class img_obj:
    """
    The basic image object
    """
    def __init__(self, filename, screen):
        self.filename = filename
        self.obj = None
        self.screen = screen
        self.x = -1
        self.y = -1
        self.mx = 0
        self.my = 0
        self.max_x = -1
        self.max_y = -1
        self.min_x = -1
        self.min_y = -1
        self.wide = -1
        self.tall = -1
        self.size = 1.0
        self.delta_grow = 0.001
        self.setup()

    def setup(self):
        """
        Load the image and set it up
        """
        self.obj = pygame.image.load(self.filename)
        self.wide, self.tall = self.obj.get_size()
        max_x, max_y = self.screen.get_size()
        self.max_x = (max_x - self.wide)
        self.max_y = (max_y - self.tall)
        self.min_x = 0 #int(self.wide/2)
        self.min_y = 0 #int(self.tall/2)
        self.randomize()

    def randomize(self):
        """
        randomize the location and direction
        """
        self.x = int(self.max_x * random.random())
        self.y = int(self.max_y * random.random())
        self.jiggle()

    def jiggle(self):
        """
        Jiggle the motion
        """
        print "OLDM X: %i\tY: %i" % (self.mx, self.my)
        self.mx = 0
        self.my = 0
        while self.mx == 0:
            self.mx = int(random.random() * 11) -5
        while self.my == 0:
            self.my = int(random.random() * 11) -5

        print "NEWM X: %i\tY: %i" % (self.mx, self.my)

    def move(self):
        """
        Move the object
        """
        self.x = self.x + self.mx
        self.y = self.y + self.my
        if self.x >= self.max_x:
            self.x = self.max_x
            self.mx = self.mx * -1
        elif self.x <= self.min_x:
            self.x = self.min_x
            self.mx = self.mx * -1

        if self.y >= self.max_y:
            self.y = self.max_y
            self.my = self.my * -1
        elif self.y <= self.min_y:
            self.y = self.min_y
            self.my = self.my * -1

        if self.size < 1.0:
            self.size = self.size + self.delta_grow

    def blit(self):
        """
        blit object
        """
        if self.size < 0.0:
            self.size = 0.5

        if self.size >= 1.0:
            self.screen.blit(self.obj, [self.x, self.y])
        else:
            tempimg = pygame.transform.rotozoom(self.obj, 0.0, self.size)
            self.screen.blit(tempimg, [self.x, self.y])

class FatDudeInTheJungle:
    def __init__(self, max_x, max_y):
        self.max_x = max_x
        self.max_y = max_y
        self.images = []
        self.screen = None
        self.asplode = None
        self.angle = 1
        self.scale = 0.995
        self.max_angle = 10
        self.min_angle = -10
        self.ax = 0.1
        self.fix = False
        self.game = 0
        self.startup()

    def startup(self):
        """
        """
        pygame.init()
        size = self.max_x, self.max_y
        self.screen = pygame.display.set_mode(size)
        pygame.display.set_caption("fgijagotchi - 'r' to randomize variant, 'f' to \"fix\", '1' '2' or '3' to cycle songs, left click to jiggle")

        load = pygame.image.load("image/%s" % loading_file)
        self.screen.fill((0,0,0))
        self.screen.blit(load, [0,100])
        pygame.display.flip()

        pygame.mixer.init()
        self.music_object = pygame.mixer.Sound("audio/%s" % music)
        self.giggle_object = pygame.mixer.Sound("audio/%s" % giggle)
        self.music_object.play(-1)

        for i in image_files:
            filename = "image/%s" % i
            self.images.append(img_obj(filename, self.screen))

        self.asplode = pygame.image.load("image/explode.png")

    def do_your_thang(self, variant):
        """
        My ass hurts
        """
        if variant == 1:
            tempscr = pygame.transform.rotozoom(self.screen, self.angle, self.scale)
            w2, h2 = tempscr.get_size()
            ox = int((self.max_x - w2)/2)
            oy = int((self.max_y - h2)/2)
            self.screen.blit(tempscr, [ox, oy])
        elif variant == 2:
            tempscr = pygame.transform.rotozoom(self.screen, 0.0, 0.99)
            w2, h2 = tempscr.get_size()
            ox = int((self.max_x - w2)/2)
            oy = int((self.max_y - h2)/2)
            self.screen.blit(tempscr, [ox,oy])
        elif variant == 3:
            tempscr = pygame.transform.rotozoom(self.screen, 1.0, 0.99)
            w2, h2 = tempscr.get_size()
            ox = int((self.max_x - w2)/2)
            oy = int((self.max_y - h2)/2)
            self.screen.blit(tempscr, [ox,oy])
        elif variant == 4:
            tempscr = pygame.transform.rotozoom(self.screen, self.angle/5, 0.99)
            w2, h2 = tempscr.get_size()
            ox = int((self.max_x - w2)/2)
            oy = int((self.max_y - h2)/2)
            tempscr.set_alpha(250)
            self.screen.fill((255*random.random(),255*random.random(),255*random.random()))
            self.screen.blit(tempscr, [ox,oy])
        elif variant == 5:
            tempscr = pygame.transform.rotozoom(self.screen, 1.0, 1.01)
            w2, h2 = tempscr.get_size()
            ox = int((self.max_x - w2)/2)
            oy = int((self.max_y - h2)/2)
            self.screen.blit(tempscr, [ox,oy])
        elif variant == 6:
            tempscr = pygame.transform.rotozoom(self.screen, self.angle/5, 1.01)
            w2, h2 = tempscr.get_size()
            ox = int((self.max_x - w2)/2)
            oy = int((self.max_y - h2)/2)
            tempscr.set_alpha(250)
            self.screen.fill((255*random.random(),255*random.random(),255*random.random()))
            self.screen.blit(tempscr, [ox,oy])
        else:
            tempscr = pygame.transform.rotozoom(self.screen, self.angle, self.scale)
            self.screen.blit(tempscr, [-10,-10])

        self.angle = self.angle + self.ax
        if self.angle > self.max_angle:
            self.angle = self.max_angle
            self.ax = self.ax * -1
        elif self.angle < self.min_angle:
            self.angle = self.min_angle
            self.ax = self.ax * -1
        pygame.display.flip()

    def collision(self):
        """
        Basic mouse collision detection
        """
        (x, y) = pygame.mouse.get_pos()
        for i in self.images:
            if (x > i.x and x < (i.x + i.wide)) and (y > i.y and y < (i.y + i.tall)):
                tempimg = pygame.transform.scale(self.asplode, (i.wide, i.tall))
                self.screen.blit(tempimg, [i.x, i.y])
                i.randomize()
                i.size = 0.01
                i.delta_grow = 0.01

    def eval_games(self, variant):
        # There's some variants that are special settings of the
        # others
        if variant == 7:
            self.game = 1
            variant = 3
            self.fix = True
        else:
            self.game = 0

        return variant

    def run_fatty(self, variant):
        """
        Run fatty!
        """
        variant = self.eval_games(variant)
        while True:
            for i in self.images:
                if not self.fix:
                    self.do_your_thang(variant)
                i.move()
                i.blit()

            if self.fix:
                self.do_your_thang(variant)

            for event in pygame.event.get():
                if event.type == QUIT:
                    #self.music_object.stop()
                    return
                elif event.type == KEYDOWN and event.key == K_ESCAPE:
                    #self.music_object.stop()
                    return
                elif event.type == KEYDOWN and event.key == K_r:
                    variant = int(len(variants) * random.random())
                    print "New variant is %i \"%s\", bitch!" % (variant, variants[variant])
                    variant = self.eval_games(variant)
                elif event.type == KEYDOWN and event.key == K_f and self.game == 0:
                    self.fix = not self.fix
                    print "Toggling the \"fix\"... damnit..."
		elif event.type == KEYDOWN and event.key == K_1:
		    self.music_object.stop()
		    self.music_object = pygame.mixer.Sound("audio/popcorn.ogg")
		    self.music_object.play(-1)
		    print "New song is Popcorn, bitch!"
		elif event.type == KEYDOWN and event.key == K_2:
		    self.music_object.stop()
                    self.music_object = pygame.mixer.Sound("audio/uspinmerr.ogg")
		    self.music_object.play(-1)
		    print "New song is You Spin Me Right Round, bitch!"
		elif event.type == KEYDOWN and event.key == K_3:
		    self.music_object.stop()
                    self.music_object = pygame.mixer.Sound("audio/spinaround.ogg")
		    self.music_object.play(-1)
		    print "New song is Spin Around, bitch!"
                elif event.type == MOUSEBUTTONDOWN:
                    print "Jiggle them titties"
                    self.giggle_object.play()
                    if self.game: self.collision()
                    for i in self.images:
                        i.jiggle()

def usage():
    print "fgijagotchi usage"
    print
    print " fgijagotchi [OPTION]"
    print "   where [OPTION] is one of the following"
    print
    for i in range(len(variants)):
        print "\t%i\t%s" % (i, variants[i])
    print "\n"

def main():
    usage()
    variant = 0
    if len(sys.argv) > 1:
        if sys.argv[1].isdigit():
            variant = int(sys.argv[1])

    print "variant : %i" % variant
    fatguy = FatDudeInTheJungle(640, 480)
    fatguy.run_fatty(variant)

if __name__ == "__main__":
    main()
